@@ -292,7 +292,8 @@ class BlockRecurrentTransformer(nn.Module):
         xl_memories_layers: Optional[Tuple[int, ...]] = None,
         recurrent_layers: Optional[Tuple[int, ...]] = None,
         num_state_vectors = 512,
-        dynamic_pos_bias_dim = None
+        dynamic_pos_bias_dim = None,
+        enhanced_recurrence = False
     ):
         super().__init__()
         xl_memories_layers = default(xl_memories_layers, tuple(range(1, depth + 1)))
@@ -344,6 +345,8 @@ class BlockRecurrentTransformer(nn.Module):
         self.max_seq_len = max_seq_len
         self.ignore_index = ignore_index
 
+        self.enhanced_recurrence = enhanced_recurrence
+
     @torch.no_grad()
     @eval_decorator
     def generate(
@@ -372,8 +375,8 @@ class BlockRecurrentTransformer(nn.Module):
         self,
         x,
         return_loss = False,
-        xl_memories: List[torch.Tensor] = tuple(),
-        states: List[torch.Tensor] = tuple(),
+        xl_memories: List[torch.Tensor] = [],
+        states: List[torch.Tensor] = [],
 
     ):
         device = x.device
@@ -403,6 +406,11 @@ class BlockRecurrentTransformer(nn.Module):
         pos_bias = pos_bias[rel_pos]
         pos_bias = rearrange(pos_bias, 'i j h -> h i j')
 
+        # enhanced recurrence
+
+        if self.enhanced_recurrence and len(xl_memories) > 1:
+            xl_memories = [*xl_memories[1:], xl_memories[0]]
+
         # ready xl memories and states
 
         xl_memories = iter(xl_memories)
